@@ -34,7 +34,6 @@ from kineticstoolkit.exceptions import (
     TimeSeriesEventNotFoundError,
 )
 import warnings
-from typing import Optional
 from numpy.typing import ArrayLike
 
 
@@ -124,7 +123,6 @@ def detect_cycles(
     is_phase1 = True
 
     for i in range(time.shape[0]):
-
         if directions[0] == "rising":
             crossing1 = data[i] >= thresholds[0]
             crossing2 = data[i] <= thresholds[1]
@@ -133,12 +131,10 @@ def detect_cycles(
             crossing2 = data[i] >= thresholds[1]
 
         if is_phase1 and crossing1:
-
             is_phase1 = False
             events.append(TimeSeriesEvent(time[i], event_names[0]))
 
         elif (not is_phase1) and crossing2:
-
             is_phase1 = True
             events.append(TimeSeriesEvent(time[i], event_names[1]))
 
@@ -198,7 +194,7 @@ def time_normalize(
     event_name2: str,
     *,
     n_points: int = 100,
-    span: Optional[list[int]] = None,
+    span: list[int] | None = None,
 ) -> TimeSeries:
     """
     Time-normalize cycles in a TimeSeries.
@@ -251,9 +247,18 @@ def time_normalize(
     if span is None:
         span = [0, n_points]
 
-    # Find the final number of cycles
     if len(ts.events) < 2:
-        raise (ValueError("No cycle can be defined from these event names."))
+        raise ValueError("This TimeSeries does not have events.")
+    if ts.count_events(event_name1) == 0:
+        raise ValueError(
+            f"No occurrence of event `{event_name1}` was found in this "
+            "TimeSeries."
+        )
+    if ts.count_events(event_name2) == 0:
+        raise ValueError(
+            f"No occurrence of event `{event_name2}` was found in this "
+            "TimeSeries."
+        )
 
     # Initialize the destination TimeSeries
     dest_ts = ts.copy()
@@ -270,7 +275,6 @@ def time_normalize(
     i_cycle = 0
     break_now = False
     while True:
-
         # Get the begin time for this cycle
         try:
             event_index = ts._get_event_index(event_name1, i_cycle)
@@ -279,6 +283,9 @@ def time_normalize(
         else:
             begin_time = ts.events[event_index].time
 
+        if break_now:
+            break
+
         # Get the end time for this cycle
         end_cycle = 0
         end_time = ts.events[ts._get_event_index(event_name2, end_cycle)].time
@@ -373,7 +380,6 @@ def time_normalize(
             ) * (span[1] - span[0]) + i_cycle * (span[1] - span[0])
 
         for i_event, event in enumerate(other_events):
-
             # Resample
             new_time = time_to_normalized_time(event.time)
             dest_ts = dest_ts.add_event(new_time, event.name)
@@ -445,7 +451,7 @@ def stack(ts: TimeSeries, *, n_points: int = 100) -> dict[str, np.ndarray]:
     return data
 
 
-def unstack(data: dict[str, ArrayLike], /) -> TimeSeries:
+def unstack(data: dict[str, np.ndarray], /) -> TimeSeries:
     """
     Unstack time-normalized data from a dict of arrays to a TimeSeries.
 
@@ -610,7 +616,6 @@ def most_repeatable_cycles(data: ArrayLike, /) -> list[int]:
     # Iteratively remove the cycle that is the most different from the
     # mean of the remaining cycles.
     while len(done_cycles) < n_cycles - 2:
-
         current_mean_cycle = np.nanmean(data, axis=0)
 
         rms = np.zeros(n_cycles)
