@@ -37,6 +37,7 @@ import kineticstoolkit.config
 import os
 import numpy as np
 import pandas as pd
+from typing import Any
 import warnings
 import shutil
 import json
@@ -45,8 +46,6 @@ import time
 import getpass
 import zipfile
 
-from typing import Any
-
 
 def __dir__():  # pragma: no cover
     return ["save", "load"]
@@ -295,19 +294,30 @@ def load(filename: str, *, include_metadata: bool = False) -> Any:
 def read_c3d(
     filename: str,
     *,
-    convert_point_unit: bool = True,
+    convert_point_unit: bool | None = None,
     **kwargs,
 ) -> dict[str, TimeSeries]:
     """
     Read point and analog data from a C3D file.
 
-    Point positions are returned in `output['Points']` as a TimeSeries, where
+    Point positions are returned in `output["Points"]` as a TimeSeries, where
     each point corresponds to a data key. Each point position is expressed as
-    an Nx4 point series.
+    an Nx4 point series::
+        [
+            [x0, y0, z0, 1.0],
+            [x1, y1, z1, 1.0],
+            [x2, y2, z2, 1.0],
+            ...,
+        ]
 
-    If available, analog data is returned in `output['Analogs']` as a
+    If available, analog data is returned in `output["Analogs"]` as a
     TimeSeries, where each analog signal is expressed as a unidimensional
-    series of length N.
+    array of length N.
+
+    Some applications store calculated values such as angles, forces, moments,
+    powers, etc. into the C3D file. Storing these data is application-specific
+    and is not standardized in the C3D file format (https://www.c3d.org).
+    This function reads these values as points regardless of their nature.
 
     Parameters
     ----------
@@ -315,15 +325,34 @@ def read_c3d(
         Path of the C3D file
 
     convert_point_unit
-        Optional. True to convert the point units to meters, even if they are
-        expressed in mm in the C3D file.
+        Optional. True to convert the point units to meters, if they are
+        expressed in other units such as mm in the C3D file. False to keep
+        points as is. When unset, if points are stored in a unit other than
+        meters, then a warning is issued. See caution note below.
 
     Returns
     -------
-    Dict of TimeSeries
+    dict[ktk.TimeSeries]
         A dict of TimeSeries, with keys being "Points" and if available,
         "Analogs".
 
+    Caution
+    -------
+    If, for a given C3D file, points are expressed in another unit than meters
+    (e.g., mm), and that this file also contains calculated points such as
+    angles, powers, etc., then you need to be cautious with the
+    `convert_point_unit` parameter:
+        
+    - Setting `convert_point_unit` to False reads the file as is, but you
+      then need to manually convert the points to meters. This can be done
+      easily using ktk.geometry.scale.
+    - Setting `convert_point_unit` to True scales all points to meters,
+      but also scales every calculated angle, power, etc. as they are read
+      as any other point.
+
+    For these special cases, we recommend to set `convert_point_unit` to
+    False, and then scale the points manually.
+
     Warning
     -------
     This function, which has been introduced in version 0.9, is still
@@ -331,12 +360,12 @@ def read_c3d(
 
     See also
     --------
-    ktk.write_c3d
+    ktk.write_c3d, ktk.geometry.scale
 
     Notes
     -----
-    - This function relies on `ezc3d`, which is available on
-      conda-forge and on git-hub. Please install ezc3d before using
+    - This function relies on `ezc3d`, which is installed by default using
+      conda, but not using pip. Please install ezc3d before using
       read_c3d. https://github.com/pyomeca/ezc3d
 
     - As for any instrument, please check that your data loads correctly on
@@ -438,14 +467,53 @@ def read_c3d(
     labels = reader["parameters"]["POINT"]["LABELS"]["value"]
     n_points = reader["parameters"]["POINT"]["USED"]["value"][0]
 
-    if convert_point_unit and (point_unit == "mm"):
-        point_factor = 0.001
-        point_unit = "m"
-    elif point_unit == "m":
-        point_factor = 1
+    # Solve the point unit conversion mess (issue #147)
+    scales = {"mm": 0.001, "cm": 0.01, "dm": 0.1, "m": 1.0}
+
+    if convert_point_unit is None:
+        if point_unit == "m":
+            point_factor = 1.0
+            point_unit = "m"
+        elif point_unit in scales:
+            warnings.warn(
+                "In the specified file, points are expressed in "
+                f"{point_unit}. They were automatically converted to meters "
+                f"by scaling them by {scales[point_unit]}. Please note that "
+                "if this file also contains calculated values such as "
+                "angles, powers, etc., they were also (wrongly) scaled by "
+                f"{scales[point_unit]}. Consult "
+                "https://kineticstoolkit.uqam.ca/doc/api/ktk.read_c3d.html "
+                "for more information. You can mute this warning "
+                "by explicitely setting `convert_point_unit` to either True "
+                "or False."
+            )
+            point_factor = scales[point_unit]
+            point_unit = "m"
+        else:
+            warnings.warn(
+                "In the specified file, points are expressed in "
+                f"`{point_unit}`, which is not recognized by ktk.read_c3d. They "
+                "were left as is, without attempting to convert to meters. "
+                "You can mute this warning by setting `convert_point_unit` to "
+                "False."
+            )
+            point_factor = 1.0
+            # point_unit = Do not update
+
+    elif convert_point_unit is True:
+        try:
+            point_factor = scales[point_unit]
+            point_unit = "m"
+        except KeyError:
+            raise ValueError(
+                "In the specified file, points are expressed in "
+                f"`{point_unit}`, which is not recognized by ktk.read_c3d. "
+                "Please set `convert_point_unit` to None of False."
+            )
+
     else:
         point_factor = 1
-        warnings.warn(f"Point unit is {point_unit} instead of meters.")
+        # point_unit = Do not update
 
     for i_label in range(n_points):
         # Make sure it's UTF8, and strip leading and ending spaces
@@ -506,12 +574,10 @@ def read_c3d(
     # ---------------------------------
     # Create the platforms TimeSeries
     if extract_force_plates and reader["data"]["platform"] != []:
-
         platforms = TimeSeries(time=analogs.time)  # type: ignore
 
         n_platforms = len(reader["data"]["platform"])
         for i_platform in range(n_platforms):
-
             force_unit = reader["data"]["platform"][0]["unit_force"]
 
             if force_unit != "N":
@@ -557,7 +623,9 @@ def read_c3d(
     return output
 
 
-def write_c3d(filename: str, points: TimeSeries, **kwargs) -> None:
+def write_c3d(
+    filename: str, points: TimeSeries, analogs: TimeSeries | None = None
+) -> None:
     """
     Write points and analog data to a C3D file.
 
@@ -567,38 +635,58 @@ def write_c3d(filename: str, points: TimeSeries, **kwargs) -> None:
         Path of the C3D file
 
     points
-        Points trajectories, where each point corresponds to a data key.
-        Each point position is expressed as an Nx4 point series. Events from
-        this TimeSeries are also added to the c3d.
+        Points trajectories, where data key corresponds to a point, expressed
+        as an Nx4 point series::
+            [
+                [x0, y0, z0, 1.0],
+                [x1, y1, z1, 1.0],
+                [x2, y2, z2, 1.0],
+                ...,
+            ]
+        Events from this TimeSeries are also added to the c3d.
+
+    analogs
+        Optional. Analog signals, where each data key is one series. Series
+        that are not unidimensional are converted to multiple unidimensional
+        series. For instance, if the shape of analogs.data['Forces'] is
+        1000x3, then three unidimensional series of length 1000 are created in
+        the C3D: Forces[0], Forces[1] and Forces[2].
+
+        The sample rate of `analogs` must be an integer multiple of the
+        `points`'s sample rate. Also, `analogs.time[0]` must be the same as
+        `points.time[0]`.
 
     See also
     --------
     ktk.read_c3d
 
-    Note
-    ----
-    This function relies on `ezc3d`, which is available on
-    conda-forge and on git-hub. Please install ezc3d before using
+    Notes
+    -----
+    This function relies on `ezc3d`, which is installed by default using
+    conda, but not using pip. Please install ezc3d before using
     write_c3d. https://github.com/pyomeca/ezc3d
 
-    """
-    """
-    Additional parameters in development
-    ------------------------------------
-    analogs
-        Optional. Analog signals, where each data key is an unidimensional
-        array. Events from this TimeSeries are not added to the c3d.
-        
-    This is not released yet because more checking is required to match sizes,
-    sampling rates, etc. with useful warnings or error messages.
-    """
-    check_types(write_c3d, locals())
+    Example
+    -------
+    Create a simple c3d file with two markers sampled at 240 Hz and two
+    sinusoidal analog signals sampled at 1200 Hz, during 10 seconds::
 
-    if "analogs" in kwargs:
-        analogs = kwargs["analogs"]
-    else:
-        analogs = None
+        import kineticstoolkit.lab as ktk
+        import numpy as np
 
+        points = ktk.TimeSeries()
+        points.time = np.linspace(0, 10, 10*240, endpoint=False)
+        points.data["Marker1"] = np.ones((2400, 4))
+        points.data["Marker2"] = np.ones((2400, 4))
+
+        analogs = ktk.TimeSeries()
+        analogs.time = np.linspace(0, 10, 10*2400, endpoint=False)
+        analogs.data["Signal1"] = np.sin(analogs.time)
+        analogs.data["Signal2"] = np.cos(analogs.time)
+
+        ktk.write_c3d("testfile.c3d", points=points, analogs=analogs)
+
+    """
     try:
         import ezc3d
     except ModuleNotFoundError:
@@ -608,50 +696,104 @@ def write_c3d(filename: str, points: TimeSeries, **kwargs) -> None:
             "conda install -c conda-forge ezc3d"
         )
 
+    # Basic type check
+    check_types(write_c3d, locals())
+
     # Create an empty c3d structure
     c3d = ezc3d.c3d()
 
-    # Add the points
-    marker_list = []
-    marker_data = np.zeros((4, len(points.data), len(points.time)))
+    # Add the points, but first make some checks
+    points._check_not_empty_data()
+    points._check_constant_sample_rate()
 
-    for i_marker, marker in enumerate(points.data):
-        marker_list.append(marker)
-        marker_data[0, i_marker, :] = points.data[marker][:, 0]
-        marker_data[1, i_marker, :] = points.data[marker][:, 1]
-        marker_data[2, i_marker, :] = points.data[marker][:, 2]
-        marker_data[3, i_marker, :] = points.data[marker][:, 3]
+    point_rate = points.get_sample_rate()
+
+    point_unit = None
+    for key in points.data:
+        # Check that this is a series of points
+        if points.data[key].shape[1] != 4:
+            raise ValueError(f"Point {key} is not a Nx4 series of points.")
+        if not np.all(np.isclose(points.data[key][:, 3], 1.0)):
+            raise ValueError(f"Point {key} is not a series of [x, y, z, 1.0]")
+
+        # Check that units are all the same (or None)
+        if key not in points.data_info:
+            continue
+
+        if "Unit" not in points.data_info[key]:
+            continue
+
+        this_unit = points.data_info[key]["Unit"]
+
+        if this_unit is None:
+            continue
+
+        if point_unit is None:
+            point_unit = this_unit
+            continue
+
+        if point_unit != this_unit:
+            raise ValueError(
+                "Found different point units in the TimeSeries: "
+                f"{point_unit} and {this_unit}."
+            )
+
+    if point_unit is None:
+        point_unit = "m"  # Default
+
+    # Now format and add the points
+    point_list = []
+    point_data = np.zeros((4, len(points.data), len(points.time)))
+
+    for i_point, point in enumerate(points.data):
+        point_list.append(point)
+        point_data[0, i_point, :] = points.data[point][:, 0]
+        point_data[1, i_point, :] = points.data[point][:, 1]
+        point_data[2, i_point, :] = points.data[point][:, 2]
+        point_data[3, i_point, :] = points.data[point][:, 3]
 
     # Fill point data
-    c3d["header"]["points"]["first_frame"] = round(
-        points.time[0] * points.get_sample_rate()
-    )
-    c3d.add_parameter("POINT", "RATE", [points.get_sample_rate()])
-    c3d.add_parameter("POINT", "LABELS", [tuple(marker_list)])
-    c3d.add_parameter("POINT", "UNITS", "m")
+    c3d["header"]["points"]["first_frame"] = round(points.time[0] * point_rate)
+    c3d.add_parameter("POINT", "RATE", [point_rate])
+    c3d.add_parameter("POINT", "LABELS", [tuple(point_list)])
+    c3d.add_parameter("POINT", "UNITS", point_unit)
 
-    c3d["data"]["points"] = marker_data
+    c3d["data"]["points"] = point_data
 
     # Fill analog data
     if analogs is not None:
+        analogs._check_not_empty_data()
+        analogs._check_constant_sample_rate()
+        analog_rate = analogs.get_sample_rate()
 
-        c3d.add_parameter("ANALOG", "LABELS", list(analogs.data.keys()))
-        c3d.add_parameter("ANALOG", "RATE", [analogs.get_sample_rate()])
+        rate_ratio = analog_rate / point_rate
+        if ~np.isclose(rate_ratio, int(rate_ratio)):
+            raise ValueError(
+                "The sample rate of analogs must be an integer "
+                "multiple of the points sample rate."
+            )
+
+        if ~np.isclose(analogs.time[0], points.time[0]):
+            raise ValueError(
+                "Points and analogs must share the same starting time. "
+                f"However, points.time[0] = {points.time[0]} whereas "
+                f"analogs.time[0] = {analogs.time[0]}."
+            )
+
+        # Since analogs are unidimensional, we will use the DataFrame exporter
+        # to get one column per analog value. This way, forces would become
+        # forces[0], forces[1], forces[2] and forces[3].
+        df_analogs, analogs_data_info = analogs._to_dataframe_and_info()
+
+        c3d.add_parameter("ANALOG", "LABELS", list(df_analogs.columns))
+        c3d.add_parameter("ANALOG", "RATE", [analog_rate])
         c3d.add_parameter(
             "ANALOG",
             "UNITS",
-            [
-                analogs.data_info[key]["Unit"]
-                if key in analogs.data_info
-                and "Unit" in analogs.data_info[key]
-                else ""
-                for key in analogs.data
-            ],
+            [_["Unit"] if "Unit" in _ else "" for _ in analogs_data_info],
         )
-        c3d.add_parameter("ANALOG", "USED", [len(analogs.data)])
-
-        analog_data = np.array([analogs.data[key] for key in analogs.data])
-        c3d["data"]["analogs"] = analog_data[np.newaxis]
+        c3d.add_parameter("ANALOG", "USED", [len(analogs_data_info)])
+        c3d["data"]["analogs"] = (df_analogs.to_numpy().T)[np.newaxis]
 
     # Write the data
     c3d.write(filename)
