@@ -114,7 +114,6 @@ def savgol(
     See also
     --------
     ktk.filters.smooth
-    ktk.filters.median
 
     """
     check_types(savgol, locals())
@@ -125,7 +124,6 @@ def savgol(
     delta = ts.time[1] - ts.time[0]
 
     for key in tsout.data.keys():
-
         (subts, nan_index) = _interpolate(tsout, key)
 
         if np.sum(~nan_index) < poly_order + 1:
@@ -180,7 +178,6 @@ def smooth(ts: TimeSeries, /, window_length: int) -> TimeSeries:
     See also
     --------
     ktk.filters.savgol
-    ktk.filters.median
 
     """
     check_types(smooth, locals())
@@ -250,7 +247,6 @@ def butter(
     sos = sgl.butter(order, fc, btype, analog=False, output="sos", fs=fs)
 
     for data in ts.data:
-
         (subts, missing) = _interpolate(ts, data)
 
         # Filter
@@ -353,11 +349,6 @@ def median(ts: TimeSeries, /, window_length: int = 3) -> TimeSeries:
     window_length
         Optional. Kernel size, must be odd. The default is 3.
 
-    See also
-    --------
-    ktk.butter.smooth
-    ktk.butter.savgol
-
     Example
     -------
     >>> ts = ktk.TimeSeries(time=np.arange(0, 0.5, 0.1))
