@@ -43,18 +43,20 @@ def __dir__():
         "get_global_coordinates",
         "isnan",
         "register_points",
+        "rotate",
+        "translate",
+        "scale",
     ]
 
 
 import numpy as np
 import scipy.spatial.transform as transform
 import kineticstoolkit.external.icp as icp
-from typing import Optional
 from numpy.typing import ArrayLike
 from kineticstoolkit.exceptions import check_types
 
 
-def matmul(op1: ArrayLike, op2: ArrayLike) -> np.ndarray:
+def matmul(op1: ArrayLike, op2: ArrayLike, /) -> np.ndarray:
     """
     Matrix multiplication between series of matrices.
 
@@ -114,14 +116,70 @@ def matmul(op1: ArrayLike, op2: ArrayLike) -> np.ndarray:
     return result
 
 
+def inv(matrix_series: ArrayLike, /) -> np.ndarray:
+    """
+    Calculate series of inverse transform.
+
+    This function calculates a series of inverse homogeneous transforms.
+
+    Parameters
+    ----------
+    matrix_series
+        Nx4x4 series of homogeneous matrices, where each matrix is an
+        homogeneous transform.
+
+    Returns
+    -------
+    ArrayLike
+        The Nx4x4 series of inverse homogeneous matrices.
+
+    Note
+    ----
+    This function requires (and checks) that each matrix really is an
+    homogeneous transform by evaluating the determinant of its rotation
+    component. It then calculates the inverse matrix quickly using the
+    transpose of the rotation component.
+
+    """
+    check_types(inv, locals())
+
+    matrix_series = np.array(matrix_series)
+    index_is_nan = isnan(matrix_series)
+
+    _check_no_skewed_rotation(matrix_series, "matrix_series")
+
+    invR = np.zeros((matrix_series.shape[0], 3, 3))
+    invT = np.zeros((matrix_series.shape[0], 3))
+
+    # Inverse rotation
+    invR = np.transpose(matrix_series[:, 0:3, 0:3], (0, 2, 1))
+
+    # Inverse translation
+    invT = -matmul(invR, matrix_series[:, 0:3, 3])
+
+    # output
+    out = np.zeros(matrix_series.shape)
+    out[:, 0:3, 0:3] = invR
+    out[:, 0:3, 3] = invT
+    out[:, 3, 3] = 1
+    out[index_is_nan] = np.nan
+
+    return out
+
+
 def create_transforms(
-    seq: Optional[str] = None,
-    angles: Optional[ArrayLike] = None,
-    translations: Optional[ArrayLike] = None,
+    seq: str | None = None,
+    angles: ArrayLike | None = None,
+    translations: ArrayLike | None = None,
+    scales: ArrayLike | None = None,
+    *,
     degrees=False,
 ) -> np.ndarray:
     """
-    Create an Nx4x4 series of homogeneous transforms based on angle sequences.
+    Create series of transforms based on angles, translatations and scales.
+
+    Create an Nx4x4 series of homogeneous transform matrices based on series of
+    angles, translatations and scales.
 
     Parameters
     ----------
@@ -149,10 +207,14 @@ def create_transforms(
         angles and W is the length of `seq`.
 
     translations
-        Optional array_like or shape (N, 3) or (N, 4). This corresponds
+        Optional array_like of shape (N, 3) or (N, 4). This corresponds
         to the translation part of the generated series of homogeneous
         transforms.
 
+    scales
+        Optional array_like of shape (N, ) that corresponds to the scale to
+        apply uniformly on the three axes. By default, no scale is included.
+
     degrees
         If True, then the given angles are in degrees. Default is False.
 
@@ -163,15 +225,16 @@ def create_transforms(
 
     See also
     --------
-    ktk.geometry.create_frames
+    ktk.geometry.create_frames, ktk.geometry.rotate, ktk.geometry.translate,
+    ktk.geometry.scale
 
-    Example
-    -------
+    Examples
+    --------
     Create a series of two homogeneous transforms that rotates 0, then 90
     degrees around x:
 
         >>> import kineticstoolkit.lab as ktk
-        >>> ktk.geometry.create_transforms('x', [0, 90], degrees=True)
+        >>> ktk.geometry.create_transforms(seq='x', angles=[0, 90], degrees=True)
         array([[[ 1.,  0.,  0.,  0.],
                 [ 0.,  1.,  0.,  0.],
                 [ 0.,  0.,  1.,  0.],
@@ -182,6 +245,15 @@ def create_transforms(
                 [ 0.,  1.,  0.,  0.],
                 [ 0.,  0.,  0.,  1.]]])
 
+    Create an homogeneous transform that converts millimeters to meters
+
+        >>> import kineticstoolkit.lab as ktk
+        >>> ktk.geometry.create_transforms(scales=[0.001])
+        array([[[0.001, 0.   , 0.   , 0.   ],
+                [0.   , 0.001, 0.   , 0.   ],
+                [0.   , 0.   , 0.001, 0.   ],
+                [0.   , 0.   , 0.   , 1.   ]]])
+
     """
     check_types(create_transforms, locals())
 
@@ -198,8 +270,24 @@ def create_transforms(
     else:
         angles = np.array(angles)
 
+    # Condition scales
+    if scales is None:
+        scales = np.array([1])
+    else:
+        scales = np.array(scales)
+
+    # Convert scales to a series of scaling matrices
+    temp = np.zeros((scales.shape[0], 4, 4))
+    temp[:, 0, 0] = scales
+    temp[:, 1, 1] = scales
+    temp[:, 2, 2] = scales
+    temp[:, 3, 3] = 1.0
+    scales = temp
+
     # Match sizes
     translations, angles = _match_size(translations, angles)
+    translations, scales = _match_size(translations, scales)
+    translations, angles = _match_size(translations, angles)
     n_samples = angles.shape[0]
 
     # Create the rotation matrix
@@ -208,13 +296,157 @@ def create_transforms(
     if len(R.shape) == 2:  # Single rotation: add the Time dimension.
         R = R[np.newaxis, ...]
 
-    # Construct the final series of transforms
+    # Construct the final series of transforms (without scaling)
     T = np.empty((n_samples, 4, 4))
     T[:, 0:3, 0:3] = R
     T[:, 0:3, 3] = translations
     T[:, 3, 0:3] = 0
     T[:, 3, 3] = 1
-    return T
+
+    # Return the scaling + transform
+    return T @ scales
+
+
+def rotate(
+    coordinates, /, seq: str, angles: ArrayLike, *, degrees: bool = False
+) -> np.ndarray:
+    """
+    Rotate a series of coordinates along given axes.
+
+    Parameters
+    ----------
+    coordinates
+        Array_like of shape (N, ...): the coordinates to rotate.
+
+    seq
+        Specifies sequence of axes for rotations. Up to 3 characters
+        belonging to the set {'X', 'Y', 'Z'} for intrinsic rotations (moving
+        axes), or {'x', 'y', 'z'} for extrinsic rotations (fixed axes).
+        Extrinsic and intrinsic rotations cannot be mixed in one function call.
+
+    angles
+        Array_like of shape (N,) or (N, [1 or 2 or 3]). Angles are
+        specified in radians (if degrees is False) or degrees (if degrees is
+        True).
+
+        For a single-character `seq`, `angles` can be:
+
+        - array_like with shape (N,), where each `angle[i]` corresponds to a
+          single rotation;
+        - array_like with shape (N, 1), where each `angle[i, 0]` corresponds
+          to a single rotation.
+
+        For 2- and 3-character `seq`, `angles` is an array_like with shape
+        (N, W) where each `angle[i, :]` corresponds to a sequence of Euler
+        angles and W is the length of `seq`.
+
+    degrees
+        If True, then the given angles are in degrees. Default is False.
+
+    Returns
+    -------
+    np.ndarray
+        Array_like of shape (N, ...): the rotated coordinates.
+
+    See also
+    --------
+    ktk.geometry.translate, ktk.geometry.scale, ktk.geometry.create_transforms,
+    ktk.geometry.matmul
+
+    Examples
+    --------
+    Rotate the point (1, 0, 0) by theta degrees around z, then by 45 degrees
+    around y, for theta in [0, 10, 20, 30, 40]:
+
+        >>> import kineticstoolkit.lab as ktk
+        >>> angles = np.array([[0, 45], [10, 45], [20, 45], [30, 45], [40, 45]])
+        >>> ktk.geometry.rotate([[1, 0, 0, 1]], 'zx', angles, degrees=True)
+        array([[1.        , 0.        , 0.        , 1.        ],
+               [0.98480775, 0.1227878 , 0.1227878 , 1.        ],
+               [0.93969262, 0.24184476, 0.24184476, 1.        ],
+               [0.8660254 , 0.35355339, 0.35355339, 1.        ],
+               [0.76604444, 0.45451948, 0.45451948, 1.        ]])
+
+    """
+    return matmul(create_transforms(seq, angles, degrees=degrees), coordinates)
+
+
+def translate(coordinates, /, translations):
+    """
+    Translate a series of coordinates.
+
+    Parameters
+    ----------
+    coordinates
+        Array_like of shape (N, ...): the coordinates to translate.
+
+    translations
+        Array_like of shape (N, 3) or (N, 4): the translation on each axe
+        (x, y, z).
+
+    Returns
+    -------
+    np.ndarray
+        Array_like of shape (N, ...): the translated coordinates.
+
+    See also
+    --------
+    ktk.geometry.rotate, ktk.geometry.scale, ktk.geometry.create_transforms,
+    ktk.geometry.matmul
+
+    Examples
+    --------
+    Translate the point (1, 0, 0) by (x, 1, 0), for x in [0, 1, 2, 3, 4]:
+
+        >>> import kineticstoolkit.lab as ktk
+        >>> t = np.array([[0, 1, 0], [1, 1, 0], [2, 1, 0], [3, 1, 0], [4, 1, 0]])
+        >>> ktk.geometry.translate([[1, 0, 0, 1]], t)
+        array([[1., 1., 0., 1.],
+               [2., 1., 0., 1.],
+               [3., 1., 0., 1.],
+               [4., 1., 0., 1.],
+               [5., 1., 0., 1.]])
+    """
+    return matmul(create_transforms(translations=translations), coordinates)
+
+
+def scale(coordinates, /, scales):
+    """
+    Scale a series of coordinates.
+
+    Parameters
+    ----------
+    coordinates
+        Array_like of shape (N, ...): the coordinates to scale.
+
+    scales
+        Array_like of shape (N, ) that corresponds to the scale to apply
+        uniformly on the three axes.
+
+    Returns
+    -------
+    np.ndarray
+        Array_like of shape (N, ...): the translated coordinates.
+
+    See also
+    --------
+    ktk.geometry.rotate, ktk.geometry.translate,
+    ktk.geometry.create_transforms, ktk.geometry.matmul
+
+    Examples
+    --------
+    Scale the point (1, 0, 0) by x, for x in [0, 1, 2, 3, 4]:
+
+        >>> import kineticstoolkit.lab as ktk
+        >>> s = np.array([0, 1, 2, 3, 4])
+        >>> ktk.geometry.scale([[1, 0, 0, 1]], s)
+        array([[0., 0., 0., 1.],
+               [1., 0., 0., 1.],
+               [2., 0., 0., 1.],
+               [3., 0., 0., 1.],
+               [4., 0., 0., 1.]])
+    """
+    return matmul(create_transforms(scales=scales), coordinates)
 
 
 def get_angles(
@@ -283,6 +515,9 @@ def get_angles(
     check_types(get_angles, locals())
 
     T = np.array(T)
+
+    _check_no_skewed_rotation(T, "T")
+
     R = transform.Rotation.from_matrix(T[:, 0:3, 0:3])
     angles = R.as_euler(seq, degrees)
 
@@ -304,12 +539,12 @@ def get_angles(
 
 def create_frames(
     origin: ArrayLike,
-    x: Optional[ArrayLike] = None,
-    y: Optional[ArrayLike] = None,
-    z: Optional[ArrayLike] = None,
-    xy: Optional[ArrayLike] = None,
-    xz: Optional[ArrayLike] = None,
-    yz: Optional[ArrayLike] = None,
+    x: ArrayLike | None = None,
+    y: ArrayLike | None = None,
+    z: ArrayLike | None = None,
+    xy: ArrayLike | None = None,
+    xz: ArrayLike | None = None,
+    yz: ArrayLike | None = None,
 ) -> np.ndarray:
     """
     Create an Nx4x4 series of frames based on series of points and vectors.
@@ -439,6 +674,12 @@ def get_local_coordinates(
     global_coordinates = np.array(global_coordinates)
     reference_frames = np.array(reference_frames)
 
+    _check_no_skewed_rotation(reference_frames, "reference_frames")
+
+    (global_coordinates, reference_frames) = _match_size(
+        global_coordinates, reference_frames
+    )
+
     n_samples = global_coordinates.shape[0]
 
     # Transform NaNs in global coordinates to zeros to perform the operation,
@@ -447,20 +688,7 @@ def get_local_coordinates(
     global_coordinates[nan_index] = 0
 
     # Invert the reference frame to obtain the inverse transformation
-    ref_rot = reference_frames[:, 0:3, 0:3]
-    ref_t = reference_frames[:, 0:3, 3]
-
-    # Inverse rotation : transpose.
-    inv_ref_rot = np.transpose(ref_rot, (0, 2, 1))
-
-    # Inverse translation : we inverse-rotate the translation.
-    inv_ref_t = np.zeros(ref_t.shape)
-    inv_ref_t = matmul(inv_ref_rot, -ref_t)
-
-    inv_ref_T = np.zeros((n_samples, 4, 4))  # init
-    inv_ref_T[:, 0:3, 0:3] = inv_ref_rot
-    inv_ref_T[:, 0:3, 3] = inv_ref_t
-    inv_ref_T[:, 3, 3] = np.ones(n_samples)
+    inv_ref_T = inv(reference_frames)
 
     local_coordinates = np.zeros(global_coordinates.shape)  # init
     local_coordinates = matmul(inv_ref_T, global_coordinates)
@@ -507,6 +735,12 @@ def get_global_coordinates(
     local_coordinates = np.array(local_coordinates)
     reference_frames = np.array(reference_frames)
 
+    _check_no_skewed_rotation(reference_frames, "reference_frames")
+
+    (local_coordinates, reference_frames) = _match_size(
+        local_coordinates, reference_frames
+    )
+
     global_coordinates = np.zeros(local_coordinates.shape)
     global_coordinates = matmul(reference_frames, local_coordinates)
     return global_coordinates
@@ -514,7 +748,7 @@ def get_global_coordinates(
 
 def isnan(array: ArrayLike, /) -> np.ndarray:
     """
-    Check which samples has at least one NaN.
+    Check which samples contain at least one NaN.
 
     Parameters
     ----------
@@ -565,6 +799,40 @@ def _match_size(
     return op1, op2
 
 
+def _check_no_skewed_rotation(series: np.ndarray, param_name) -> None:
+    """
+    Check if all rotation matrices are orthogonal (det=1).
+
+    Parameters
+    ----------
+    matrix_series : array of shape Nx4x4
+        The input series. Inputs of other shapes are ignored.
+    param_name
+        Name of the parameters, to use in the error message.
+
+    Raises
+    ------
+    ValueError
+        If at least one skewed rotation matrix is found in the provided series.
+
+    """
+    if (
+        len(series.shape) == 3
+        and series.shape[1] == 4
+        and series.shape[2] == 4
+    ):
+        index_is_nan = isnan(series)
+        if not np.allclose(np.linalg.det(series[~index_is_nan, 0:3, 0:3]), 1):
+            raise ValueError(
+                f"Parameter {param_name} contains at least one rotation "
+                "component that is not orthogonal. This may happen, for "
+                "instance, if you attempted to resample or filter a "
+                "homogeneous transform, which is not possible directly. If "
+                "this is the case, then consider filtering quaternions or Euler "
+                "angles instead."
+            )
+
+
 def register_points(
     global_points: ArrayLike, local_points: ArrayLike
 ) -> np.ndarray:
@@ -585,12 +853,6 @@ def register_points(
         Array of shape Nx4x4, expressing a series of 4x4 homogeneous
         transforms.
 
-    Warning
-    -------
-    This function, which has been introduced in 0.4, is still experimental and
-    may change signature or behaviour in the future. Namely, the names of the
-    parameters may change in the future.
-
     """
     check_types(register_points, locals())
 
@@ -604,7 +866,6 @@ def register_points(
     T[:, 3, 3] = np.ones(n_samples)
 
     for i_sample in range(n_samples):
-
         # Identify which global points are visible
         sample_global_points = global_points[i_sample]
         sample_global_points_missing = np.isnan(
