@@ -40,8 +40,9 @@ import numpy as np
 from numpy import sin, cos
 import time
 import copy
-from typing import Any, Optional
+from typing import Any
 from numpy.typing import ArrayLike
+import warnings
 
 # To fit the new viewpoint on selecting a new marker
 import scipy.optimize as optim
@@ -158,7 +159,6 @@ class Player:
         perspective: bool = True,
         **kwargs,
     ):
-
         # Allow older variable names
         if "segments" in kwargs and interconnections == {}:
             interconnections = kwargs["segments"]
@@ -186,7 +186,6 @@ class Player:
                         self.markers.data_info[key] = one_ts.data_info[key]
 
                 elif one_ts.data[key].shape[1:] == (4, 4):
-
                     # Add the frames
                     self.rigid_bodies.data[key] = one_ts.data[key]
                     if key in one_ts.data_info:
@@ -203,9 +202,11 @@ class Player:
                         ]
 
                 else:
-                    raise ValueError(
-                        "TimeSeries data must be of shape Nx4 "
-                        "(markers) or Nx4x4 (rigid bodies)"
+                    warnings.warn(
+                        f"The data key {key} has a shape of "
+                        f"{one_ts.data[key].shape}. However, the Player can "
+                        "only show points (Nx4) and frames Nx4x4. Therefore, "
+                        f"{key} won't be shown in the Player."
                     )
         self._select_none()
         self.last_selected_marker = ""
@@ -579,7 +580,6 @@ class Player:
 
         if n_markers > 0:
             for i_marker, marker in enumerate(markers.data):
-
                 # Get this marker's color
                 if (
                     marker in markers.data_info
@@ -613,13 +613,11 @@ class Player:
         # Draw the interconnections
         if self.interconnections is not None:
             for interconnection in self.interconnections:
-
                 coordinates = []
                 chains = self.interconnections[interconnection]["Links"]
 
                 for chain in chains:
                     for marker in chain:
-
                         try:
                             coordinates.append(interconnection_markers[marker])
                         except KeyError:
@@ -947,7 +945,6 @@ class Player:
         self._update_plots()
 
     def _on_mouse_press(self, event):  # pragma: no cover
-
         if len(self.last_selected_marker) > 0:
             self._set_new_target(
                 self.markers.data[self.last_selected_marker][
@@ -1012,10 +1009,10 @@ class Player:
 
     def to_html5(
         self,
-        start_index: Optional[int] = None,
-        stop_index: Optional[int] = None,
-        start_time: Optional[int] = None,
-        stop_time: Optional[int] = None,
+        start_index: int | None = None,
+        stop_index: int | None = None,
+        start_time: float | None = None,
+        stop_time: float | None = None,
         **kwargs,
     ):
         """
